/*
 * EMS: Estación Meteorológica Simple con SD
 * Autor: ROBOteach
 * Fecha: 15/06/2023
 * Versión: 2.0
 */

#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include <SPI.h>
#include <SD.h>

const int chipSelect = 10; // pin CS del Adaptador de Micro SD

Adafruit_BME280 bmeSensor;  // I2C

float temperature;
float pressure;
float humidity;

unsigned long meassuringTime = 1000; // Lectura de datos una vez por segundo

void setup() {
  Serial.begin(115200);
  Serial.println("EMS: Estación Meteorológica Simple v2.0");

  // Inicialización sensor BME280
  unsigned status = bmeSensor.begin(0x76);
  if (!status) {
    Serial.println("Error inicializando sensor. Ejecución detenida.");
    while (1); // HALT
  }

  // Inicialización de tarjeta SD
  Serial.print("Inicializando tarjeta SD...");

  // Comprobación de la tarjeta SD
  if (!SD.begin(chipSelect)) 
  {
    Serial.println("Tarjeta dañada o no insertada");
    while (1); // HALT
  }
  Serial.println("Tarjeta SD detectada");
}

void loop() {
  // Lectura de los valores
  temperature = bmeSensor.readTemperature();
  pressure    = bmeSensor.readPressure() / 100.0F;
  humidity    = bmeSensor.readHumidity();

  // Mostrar valores
  Serial.print("T: ");
  Serial.print(temperature);
  Serial.print(" °C / ");

  Serial.print("P: ");
  Serial.print(pressure);
  Serial.print(" hPa / ");

  Serial.print("H: ");
  Serial.print(humidity);
  Serial.println(" %");

  // Cadena de texto en donde se almacenan los valores
  String dataString = "";

  // Almacenamiento de los valores en dataString
  dataString += String(temperature);
  dataString += ",";
  dataString += String(pressure);
  dataString += ",";
  dataString += String(humidity);

  // Apertura del archivo donde se almacenan los valores
  File dataFile = SD.open("EMSdata.txt", FILE_WRITE);

  // Si el archivo está disponible se escribe en él
  if (dataFile) {
    dataFile.println(dataString);
    dataFile.close();
  }
  // Si el archivo no se abre muestra un error
  else {
    Serial.println("Error abriendo EMSdata.txt");
  }

  // Darle tiempo entre medidas
  delay(meassuringTime);
}

